@@ -23,7 +23,13 @@ def kmeans_init(X, Y, n_labels, n_hidden_states, latent_node_features=False):
     # iterate over samples
     for x, y in zip(X, Y):
         # first, get neighbor counts from nodes
-        features, edges, n_hidden = x
+        if len(x) == 3:
+            features, edges, n_hidden = x
+        elif len(x) == 4:
+            # edge features are discarded
+            features, edges, _, n_hidden = x
+        else:
+            raise ValueError("Something is fishy!")
         n_visible = features.shape[0]
         if latent_node_features:
             n_visible -= n_hidden
@@ -100,14 +106,17 @@ class LatentNodeCRF(GraphCRF):
                  latent_node_features=False):
         self.n_labels = n_labels
         self.n_hidden_states = n_hidden_states
-        n_states = n_hidden_states + n_labels
+        if n_labels is not None:
+            n_states = n_hidden_states + n_labels
+        else:
+            n_states = None
         self.latent_node_features = latent_node_features
 
         GraphCRF.__init__(self, n_states, n_features,
                           inference_method=inference_method,
                           class_weight=class_weight)
 
-    def _set_size_psi(self):
+    def _set_size_joint_feature(self):
         if None in [self.n_states, self.n_features]:
             return
 
@@ -116,8 +125,8 @@ class LatentNodeCRF(GraphCRF):
         else:
             n_input_states = self.n_labels
         self.n_input_states = n_input_states
-        self.size_psi = (n_input_states * self.n_features
-                         + self.n_states * (self.n_states + 1) / 2)
+        self.size_joint_feature = (n_input_states * self.n_features +
+                                   self.n_states * (self.n_states + 1) / 2)
 
     def initialize(self, X, Y):
         n_features = X[0][0].shape[1]
@@ -133,7 +142,8 @@ class LatentNodeCRF(GraphCRF):
         elif self.n_labels != n_labels:
             raise ValueError("Expected %d labels, got %d"
                              % (self.n_labels, n_labels))
-        self._set_size_psi()
+        self.n_states = self.n_hidden_states + n_labels
+        self._set_size_joint_feature()
         self._set_class_weight()
 
     def _get_pairwise_potentials(self, x, w):
@@ -144,7 +154,7 @@ class LatentNodeCRF(GraphCRF):
         x : tuple
             Instance Representation.
 
-        w : ndarray, shape=(size_psi,)
+        w : ndarray, shape=(size_joint_feature,)
             Weight vector for CRF instance.
 
         Returns
@@ -164,7 +174,7 @@ class LatentNodeCRF(GraphCRF):
         x : tuple
             Instance Representation.
 
-        w : ndarray, shape=(size_psi,)
+        w : ndarray, shape=(size_joint_feature,)
             Weight vector for CRF instance.
 
         Returns
@@ -251,11 +261,12 @@ class LatentNodeCRF(GraphCRF):
         y_hat_org = y_hat[:y_org.size, :self.n_labels]
         return GraphCRF.continuous_loss(self, y_org, y_hat_org)
 
-    def psi(self, x, y):
+    def joint_feature(self, x, y):
         """Feature vector associated with instance (x, y).
 
-        Feature representation psi, such that the energy of the configuration
-        (x, y) and a weight vector w is given by np.dot(w, psi(x, y)).
+        Feature representation joint_feature, such that the energy of the
+        configuration (x, y) and a weight vector w is given by
+        np.dot(w, joint_feature(x, y)).
 
         Parameters
         ----------
@@ -270,7 +281,7 @@ class LatentNodeCRF(GraphCRF):
 
         Returns
         -------
-        p : ndarray, shape (size_psi,)
+        p : ndarray, shape (size_joint_feature,)
             Feature vector associated with state (x, y).
 
         """
@@ -298,8 +309,9 @@ class LatentNodeCRF(GraphCRF):
         unaries_acc = np.dot(unary_marginals[:n_visible,
                                              :self.n_input_states].T, features)
 
-        psi_vector = np.hstack([unaries_acc.ravel(), compress_sym(pw)])
-        return psi_vector
+        joint_feature_vector = np.hstack([unaries_acc.ravel(),
+                                          compress_sym(pw)])
+        return joint_feature_vector
 
     def init_latent(self, X, Y):
         # treat all edges the same
@@ -314,7 +326,7 @@ class LatentNodeCRF(GraphCRF):
             return np.sum(self.class_weight[y])
 
 
-class EdgeFeatureLatentNodeCRF(GraphCRF):
+class EdgeFeatureLatentNodeCRF(LatentNodeCRF):
     """CRF with latent variables and edge features.
 
     Yeah that's totally not a mess.
@@ -376,19 +388,14 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         n_states = n_hidden_states + n_labels
         self.n_edge_features = n_edge_features
 
-        GraphCRF.__init__(self, n_states, n_features,
-                          inference_method=inference_method,
-                          class_weight=class_weight)
-
         if latent_node_features:
             n_input_states = n_states
         else:
             n_input_states = n_labels
 
         self.n_input_states = n_input_states
-        self.size_psi = (n_input_states * self.n_features
-                         + self.n_edge_features
-                         * n_states ** 2)
+        self.size_joint_feature = (
+            n_input_states * n_features + n_edge_features * n_states ** 2)
         self.latent_node_features = latent_node_features
 
         if symmetric_edge_features is None:
@@ -405,6 +412,23 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         self.symmetric_edge_features = symmetric_edge_features
         self.antisymmetric_edge_features = antisymmetric_edge_features
 
+        GraphCRF.__init__(self, n_states, n_features,
+                          inference_method=inference_method,
+                          class_weight=class_weight)
+
+    def _set_size_joint_feature(self):
+        if None in [self.n_states, self.n_features]:
+            return
+
+        if self.latent_node_features:
+            n_input_states = self.n_states
+        else:
+            n_input_states = self.n_labels
+        self.n_input_states = n_input_states
+        self.size_joint_feature = (
+            n_input_states * self.n_features + self.n_edge_features *
+            self.n_states ** 2)
+
     def _check_size_x(self, x):
         GraphCRF._check_size_x(self, x)
 
@@ -424,7 +448,7 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         x : tuple
             Instance Representation.
 
-        w : ndarray, shape=(size_psi,)
+        w : ndarray, shape=(size_joint_feature,)
             Weight vector for CRF instance.
 
         Returns
@@ -448,7 +472,7 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         x : tuple
             Instance Representation.
 
-        w : ndarray, shape=(size_psi,)
+        w : ndarray, shape=(size_joint_feature,)
             Weight vector for CRF instance.
 
         Returns
@@ -511,8 +535,6 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
                                self.inference_method, relaxed=False)
         if (h[:len(y)] != y).any():
             print("inconsistent h and y")
-            from IPython.core.debugger import Tracer
-            Tracer()()
             h[:len(y)] = y
         return h
 
@@ -537,11 +559,12 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         y_hat_org = y_hat[:y_org.size, :self.n_labels]
         return GraphCRF.continuous_loss(self, y_org, y_hat_org)
 
-    def psi(self, x, y):
+    def joint_feature(self, x, y):
         """Feature vector associated with instance (x, y).
 
-        Feature representation psi, such that the energy of the configuration
-        (x, y) and a weight vector w is given by np.dot(w, psi(x, y)).
+        Feature representation joint_feature, such that the energy of the
+        configuration (x, y) and a weight vector w is given by
+        np.dot(w, joint_feature(x, y)).
 
         Parameters
         ----------
@@ -556,7 +579,7 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
 
         Returns
         -------
-        p : ndarray, shape (size_psi,)
+        p : ndarray, shape (size_joint_feature,)
             Feature vector associated with state (x, y).
 
         """
@@ -596,8 +619,8 @@ class EdgeFeatureLatentNodeCRF(GraphCRF):
         unaries_acc = np.dot(unary_marginals[:n_visible,
                                              :self.n_input_states].T, features)
 
-        psi_vector = np.hstack([unaries_acc.ravel(), pw.ravel()])
-        return psi_vector
+        joint_feature_vector = np.hstack([unaries_acc.ravel(), pw.ravel()])
+        return joint_feature_vector
 
     def init_latent(self, X, Y):
         # treat all edges the same
